@@ -158,6 +158,9 @@ class EventsFrame(object):
         text = f"{nm}({', '.join(kwargs)})"
         return text
 
+    def __iter__(self):
+        return (self.get_group(key) for key in self.group_keys_unique)
+
     @property
     def df(self):
         """
@@ -500,6 +503,14 @@ class EventsFrame(object):
         # Return validated columns
         return cols
 
+    def iter_groups(self):
+        """
+        Return an iterator which will iterate through all groups in the 
+        collection, yielding each group's key as well as the associated 
+        EventsGroup.
+        """
+        return ((key, self.get_group(key)) for key in self.group_keys_unique)
+
     def build_routes(self, label='route', errors='raise'):
         """
         Build MLSRoute instances for each event based on available geometry 
@@ -830,9 +841,15 @@ class EventsFrame(object):
         other = other.copy()
 
         # Check for invalid column names
-        if self.route in other.columns:
-            raise ValueError(f"Invalid column name '{self.route}' found in "
-                "target geodataframe.")
+        if (self.route in other.columns):
+            raise ValueError(
+                f"Invalid column name '{self.route}' found in target "
+                "geodataframe.")
+        if len(set(self.keys) & set(other.columns)) > 0:
+            invalid = set(self.keys) & set(other.columns)
+            raise ValueError(
+                f"Target geodataframe contains at least one events collection "
+                f"key column name {invalid}.")
 
         # Ensure that geometries and routes are available
         if self.geom is None:
@@ -888,7 +905,7 @@ class EventsFrame(object):
                 return
         locs = joined.apply(_project, axis=1)
         joined[loc_label] = locs
-
+        # return joined # modified to return EC 7/27/2022
         # Prepare and return data
         return self.__class__(
             joined.drop(columns=[self.route]),
@@ -997,7 +1014,7 @@ class EventsFrame(object):
             A number of steps per window length. The resulting step length will 
             be equal to length / steps. For non-overlapped windows, use a steps 
             value of 1.
-        fill : {'none','cut','left','right'}, default 'cut'
+        fill : {'none','cut','left','right','extend','balance'}, default 'cut'
             How to fill a gap at the end of an event's range.
 
             Options
@@ -1014,6 +1031,9 @@ class EventsFrame(object):
             extend : the final window will be anchored on the grid defined by 
                 the step value, extending beyond the window length to the right
                 bound of the event.
+            balance : if the final range is greater than or equal to half the 
+                target range length, perform the cut method; if it is less, 
+                perform the extend method.
         
         dissolve : bool, default False
             Whether to dissolve the events dataframe before performing the 
@@ -1807,7 +1827,7 @@ class EventsCollection(EventsFrame):
         # Return retrieved column data
         return res
 
-    def merge(self, other):
+    def merge(self, other, closed=None):
         """
         Create an EventsMerge instance with this collection as the left and the 
         other collection as the right. This can then be used to retrieve 
@@ -1821,6 +1841,11 @@ class EventsCollection(EventsFrame):
             with this events collection, producing an EventsMerge instance 
             which can be used to perform various overlay operations to retrieve 
             attributes and more from the target collection.
+        closed : str {'left', 'left_mod', 'right', 'right_mod', 'both', 
+                'neither'}, optional
+            Whether intervals on the target events frame are closed on the 
+            left-side, right-side, both or neither. If not provided, will 
+            default to the already-set parameter on the events frame.
         """
         # Create merge
         em = EventsMerge(self, other)
@@ -2008,51 +2033,6 @@ class EventsCollection(EventsFrame):
                            geom=self.geom, closed=self.closed)
 
 
-###########
-# HELPERS #
-###########
-
-def get_most(arr, weights):
-    """
-    Select the item from the input array which is associated with the highest 
-    total weight from the weights array. Scores are computed by summing the 
-    weights for each unique array value.
-    
-    Parameters
-    ----------
-    arr, weights : array-like
-        Arrays of equal length of target values and weights associated with 
-        each value.
-    """
-    # Enforce numpy arrays
-    arr = np.asarray(arr)
-    weights = np.asarray(weights)
-    # Group and split sorted target array
-    sorter = np.argsort(arr)
-    unique, splitter = np.unique(arr[sorter], return_index=True)
-    splitter = splitter[1:]
-    # Split weights and aggregate
-    splits = np.split(weights[sorter], splitter)
-    scores = [x.sum() for x in splits]
-    # Return the highest scoring item
-    return unique[np.argmax(scores)]
-
-def get_mode(arr):
-    """
-    Select the item from the input array which appears most frequently.
-    
-    Parameters
-    ----------
-    arr : array-like
-        Array with target values
-    """
-    # Enforce numpy array
-    arr = np.asarray(arr)
-    # Find most frequent unique value and return
-    unique, counts = np.unique(arr, return_counts=True)
-    return unique[np.argmax(counts)]
-
-
 ####################
 # COMMON FUNCTIONS #
 ####################
